/*#
# **************************************************************
# *                  SIMUSOL PROYECT                           *
# *                                                            *
# *         Table extension for:                               *
# *         C++ Mathematical Expression Toolkit Library        *
# *         which is located in:                               *
# * URL: http://www.partow.net/programming/exprtk/index.html   *
# *                                                            *
# *   The extension was developed by                           *
# *   Diego.Saravia@gmail.com, 2016                            *
# *   and could be used in the same terms of the toolkit       *
# *   or every free license with copyleft needed.              *
# *   http://www.simusol.org                                   *
# **************************************************************
# This file is part of Table.
#
#    Table is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Table is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Table.  If not, see <http://www.gnu.org/licenses/>. */


#include <cstdio>
#include <string>
#include "exprtk.hpp"


extern "C" int interpolate(const char *, const char *, const char*, double v, int m, double *);
extern "C" int finddir(char ** dir);

 template <typename T>
   struct Table : public exprtk::igeneric_function<T>
   {
      typedef typename exprtk::igeneric_function<T>::parameter_list_t
                                                     parameter_list_t;

      Table()
      : exprtk::igeneric_function<T>("SSSTT")
      {}
     
      inline T operator()(parameter_list_t parameters)	
      {
       typedef typename exprtk::igeneric_function<T>::generic_type
                                                     generic_type;     
       typedef typename generic_type::string_view string_t;

      

       double value=*(double*) parameters[3].data;
       double dmethod=*(double*) parameters[4].data;
       int method;
       method= (int) dmethod;
      
       //     printf("PARA %d %g\n", parameters.size(),	parameters[3]);
 
     
       std::string stra=exprtk::to_str(string_t(parameters[0]));
       const char * ca=stra.c_str();

       std::string strb=exprtk::to_str(string_t(parameters[1]));
       const char * cb=strb.c_str();
      
       std::string strc=exprtk::to_str(string_t(parameters[2]));
       const char * cc=strc.c_str();
      
       double result;
       int ret=interpolate(ca,cb,cc,value,method, &result);
       if (ret <0){
               printf("ERROR interpolate Table\n");
	       exit(ret);
	       /* Don't know how to handle evaluation errors in exprtk*/
           }
       return result;
      }
   };



int main()
{
   typedef exprtk::symbol_table<double> symbol_table_t;
   typedef exprtk::expression<double>     expression_t;

   typedef exprtk::parser<double>             parser_t;

   char *dir;
   char *gdir1 =NULL;
   char *gdir2 =NULL;
   char *gdir3 =NULL;
   finddir(&dir);
 
   



   //std::string expression_string  = "Table('../data/test.txt','UNO','DOS',550.,1)";
   //ojo este error no es siquiera reportado. se ve que no encuentra prototipo
   //std::string expression_string0 = "Table(gdir1,'0','31.5',550.,2)";


   
   Table<double> mf;
   symbol_table_t symbol_table;
   symbol_table.add_function("Table",mf);
   

  int error= asprintf(&gdir1,"Table('%sdata/test.txt','UNO','DOS',550.,2)",dir);
   if (error<0){
               printf("ERROR ASPRINTF 1\n");
               exit(-1);
               }
   //printf("ASPRINTF %s\n",gdir1);

   
   std::string expression_string0 = gdir1;
   expression_t expression0;
   expression0.register_symbol_table(symbol_table);
   parser_t parser0;
   parser0.compile(expression_string0,expression0);
   double result0  = expression0.value();
   printf("Result0 SP: %10.5f\n",result0);
    
   int error1= asprintf(&gdir2,"Table('%sdata/test.txt','UNO','DOS',550.,1)",dir);
   if (error1<0){
               printf("ERROR ASPRINTF 2\n");
               exit(-1);
               }
   std::string expression_string  = gdir2;
   parser_t parser;
   expression_t expression;
   expression.register_symbol_table(symbol_table);
   parser.compile(expression_string,expression);     
   double result  = expression.value();
   printf("Result LI: %10.5f\n",result);


   int error2= asprintf(&gdir3,"Table('%sdata/test.txt','UNO','DOS',550.,2)",dir);
   if (error2<0){
               printf("ERROR ASPRINTF 3\n");
               exit(-1);
               }
   std::string expression_string2  = gdir3;
   expression_t expression2;
   expression2.register_symbol_table(symbol_table);
   parser_t parser2;
   parser2.compile(expression_string2,expression2);
   double result2 = expression2.value();
   printf("Result2 SP: %10.5f\n",result2);
   return 0;
}
